
# q-expansion of newform 8880.2.a.l, downloaded from the LMFDB on 15 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 8880
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [-1], [1], [1], [-3], [6], [1], [-6], [4], [-5], [-9], [1], [-3], [7], [0], [-1], [-2], [-3], [-4], [-8], [-12], [-4], [10], [-6], [5], [16], [-8], [-6], [-11], [3], [-8], [-2], [18], [-9], [2], [-10], [-3], [23], [-4], [-9], [16], [14], [-5], [14], [-6], [-20], [-15], [-29], [3], [12], [12], [19], [-18], [0], [-18], [21], [-32], [20], [-8], [-6], [-4], [-11], [10], [-9], [-10], [-17], [-8], [18], [-12], [16], [-31], [6], [17], [-10], [-8], [20], [33], [-2], [-2], [-22], [28], [-22], [-33], [16], [-23], [-10], [36], [-3], [-7], [6], [33], [0], [-2], [-12], [-20], [0], [-12], [-41], [16], [10], [-17], [-10], [-41], [-10], [-23], [-6], [-7], [6], [-24], [9], [14], [11], [-18], [31], [-17], [-35], [35], [0], [28], [16], [35], [-32], [6], [-5], [-11], [-34], [-47], [-2], [-38], [-29], [11], [29], [-28], [-2], [-43], [-14], [21], [-52], [0], [28], [4], [50], [24], [39], [25], [-18], [-2], [-25], [14], [-45], [-35], [-43], [57], [9], [-12], [-32], [32], [-21], [-16], [14], [59], [36], [-12], [-5], [13], [19], [-51], [-26], [-43], [-30], [54], [5], [-41], [-48], [-38], [-2], [-62], [6], [37], [20], [20], [12], [-40], [7], [31], [-34], [-49], [-41], [-14], [56], [18], [40], [-62], [60], [18], [66], [0], [52], [-10], [12], [-18], [30], [-7], [-11], [35], [-42], [14], [66], [36], [-15], [-67], [-5], [26], [18], [45], [71], [54], [-40], [38], [-3], [68], [18], [69], [16], [68], [62], [-8], [12], [13], [30], [25], [-68], [14], [18], [-2], [-3], [-2], [2], [-38], [-48], [9], [-8], [56], [27], [26], [-53], [40], [28], [-35], [54], [44], [-2], [19], [26], [66], [-5], [-46], [-18], [51], [11], [-14], [8], [56], [65], [-14], [-11], [-60], [-72], [20], [38], [-46], [-23], [38], [-13], [-46], [75], [-42], [46], [17], [-6], [-64], [77], [61], [40], [28], [22], [-11], [47], [-24], [-60], [9], [-4], [-86], [21], [-75], [26], [-34], [-42], [36], [68], [29], [-14], [-28], [-40], [-48], [-57], [-1], [-58], [70], [-8], [0], [-60], [57], [6], [-30], [20], [47], [-20], [54], [8], [-44], [-10], [-51], [-37], [-16], [32], [9], [21], [-38], [49], [6], [54], [-5], [-12], [2], [58], [70], [54], [58], [-88], [-54], [-18], [-2], [-41], [70], [14], [-4], [50], [-41], [15], [-7], [-36], [-5], [-68], [34], [62], [45], [-62], [78], [-44], [74], [-42], [-98], [-90], [-18], [23], [22], [-18], [38], [-20], [70], [-7], [66], [61], [-11], [-72], [12], [-92], [-36], [92], [63], [26], [16], [88], [-10], [81], [70], [-2], [61], [-32], [59], [40], [-30], [-55], [-4], [4], [88], [-41], [-67], [12], [-66], [-85], [-22], [42], [-30], [20], [-100], [-93], [48], [-15], [-32], [-103], [21], [-6], [-81], [-68], [-66], [-61], [-72], [30], [66], [4], [-60], [73], [-16], [3], [80], [-17], [-44], [-65], [-62], [6], [-62], [-64], [74], [27], [88], [110], [-104], [65], [-10], [30], [40], [52], [-80], [3], [59], [-98], [-72], [-46], [-39], [58], [6], [65], [33], [33], [-20], [6], [-6], [90], [93], [41], [86], [61], [-87], [-111], [-76], [-84], [-66], [52], [-4], [-68], [3], [-8], [-77], [-30], [34], [-51], [55], [-67], [77], [-12], [32], [-105], [112], [-44], [-40], [-90], [4], [92], [-6], [47], [58], [-86], [30], [-106], [-12], [-12], [26], [44], [-81], [-58], [78], [-68], [46], [8], [-27], [0], [-22], [-45]]
